
---------------------------------------------------
-- FIR-filter, Direct Form, Busmakroes, Individual coefficient components
-- 4 coefficients 
-- Author: Haakon Helsing, NTNU, June 2009
---------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;

library work;
use work.coefficients.all;
USE work.my_package.all; 
use busmacro_xc4v_pkg.all;




-------------- ENTITY -----------------------------
ENTITY my_fir IS 		--component.
    PORT (
        x: IN std_logic_vector(7 downto 0);
		clk, rst : IN std_logic;
        y: OUT std_logic_vector(23 downto 0)
        );
END my_fir;
---------------------------------------------------


-------------- ARCHITECTURE -----------------------

ARCHITECTURE rtl of my_fir IS


---- register -- bussmacro -- coefficient -- bussmacro -- 'accumulate'
-- 8 --------8 --8-------8----8---------24--3x24-----3x24--24----------  <--- antall bit paa inngangene.

TYPE registers IS ARRAY (3 downto 0) OF std_logic_vector(7 downto 0);

--signal from busmacro after register to coefficient
SIGNAL reg: registers;

--wires from regs to busmacros
signal reg_m : registers;



type coefreg is array (3 downto 0) of std_logic_vector(23 downto 0);

--signal from busmacro after coefficient
signal coeff : coefreg;

--wires from 
signal coeff_m: coefreg;






--------------------PROCESS------------------------	
BEGIN
		
	
		
	process(clk, rst)
--
--	BEGIN
--		--- reset registers ---
--		if (rst = '1') then 
--			reg <= (others => (others => '0'));
--		--- update registers ---
--		elsif (clk'EVENT and clk ='1') then
--			for k in 3 downto 1 loop 
--				reg(k) <= reg(k-1);
--			end loop;
--			reg(0) <= x;
--			
--		end if;
--	END PROCESS;
	

BEGIN
	-- reset registers ---
	if (rst = '1') then 
	for i in 3 downto 0 loop
		for j in 7 downto 0 loop
			reg(i)(j) <= '0';
			end loop;
		end loop;
		
	--- update registers ---
	elsif (clk'EVENT and clk ='1') then
	for k in 3 downto 1 loop	
			reg(k) <= reg(k-1);
		end loop;
		reg(0) <= x;
		
	end if;
	
	END PROCESS;
	
	y <= coeff(0) + coeff(1) + coeff(2) + coeff(3); 
	
--- oppretter koeffisientert -----
	
	c0:coeff_0 port map(reg_m(0),coeff_m(0));
	c1:coeff_1 port map(reg_m(1),coeff_m(1));
	c2:coeff_2 port map(reg_m(2),coeff_m(2));
	c3:coeff_3 port map(reg_m(3),coeff_m(3));

----------------------------------

--- oppretter bussmakroer --------

--- between register and coefficient, left to right
	bl2r_0:busmacro_xc4v_l2r_async_wide port map(reg(0)(0),reg(0)(1),reg(0)(2),reg(0)(3),reg(0)(4),reg(0)(5),reg(0)(6),reg(0)(7),reg_m(0)(0),reg_m(0)(1),reg_m(0)(2),reg_m(0)(3),reg_m(0)(4),reg_m(0)(5),reg_m(0)(6),reg_m(0)(7));
	bl2r_1:busmacro_xc4v_l2r_async_wide port map(reg(1)(0),reg(1)(1),reg(1)(2),reg(1)(3),reg(1)(4),reg(1)(5),reg(1)(6),reg(1)(7),reg_m(1)(0),reg_m(1)(1),reg_m(1)(2),reg_m(1)(3),reg_m(1)(4),reg_m(1)(5),reg_m(1)(6),reg_m(1)(7));
	bl2r_2:busmacro_xc4v_l2r_async_wide port map(reg(2)(0),reg(2)(1),reg(2)(2),reg(2)(3),reg(2)(4),reg(2)(5),reg(2)(6),reg(2)(7),reg_m(2)(0),reg_m(2)(1),reg_m(2)(2),reg_m(2)(3),reg_m(2)(4),reg_m(2)(5),reg_m(2)(6),reg_m(2)(7));
	bl2r_3:busmacro_xc4v_l2r_async_wide port map(reg(3)(0),reg(3)(1),reg(3)(2),reg(3)(3),reg(3)(4),reg(3)(5),reg(3)(6),reg(3)(7),reg_m(3)(0),reg_m(3)(1),reg_m(3)(2),reg_m(3)(3),reg_m(3)(4),reg_m(3)(5),reg_m(3)(6),reg_m(3)(7));


--- between coefficient and adders, right to left
   --koeff0
	br2l_0a:busmacro_xc4v_l2r_async_wide port map(coeff_m(0)(0),coeff_m(0)(1),coeff_m(0)(2),coeff_m(0)(3),coeff_m(0)(4),coeff_m(0)(5),coeff_m(0)(6),coeff_m(0)(7),coeff(0)(0),coeff(0)(1),coeff(0)(2),coeff(0)(3),coeff(0)(4),coeff(0)(5),coeff(0)(6),coeff(0)(7));
	br2l_0b:busmacro_xc4v_l2r_async_wide port map(coeff_m(0)(8),coeff_m(0)(9),coeff_m(0)(10),coeff_m(0)(11),coeff_m(0)(12),coeff_m(0)(13),coeff_m(0)(14),coeff_m(0)(15),coeff(0)(8),coeff(0)(9),coeff(0)(10),coeff(0)(11),coeff(0)(12),coeff(0)(13),coeff(0)(14),coeff(0)(15));
	br2l_0c:busmacro_xc4v_l2r_async_wide port map(coeff_m(0)(16),coeff_m(0)(17),coeff_m(0)(18),coeff_m(0)(19),coeff_m(0)(20),coeff_m(0)(21),coeff_m(0)(22),coeff_m(0)(23),coeff(0)(16),coeff(0)(17),coeff(0)(18),coeff(0)(19),coeff(0)(20),coeff(0)(21),coeff(0)(22),coeff(0)(23));
	
	--koeff1:
	br2l_1:busmacro_xc4v_l2r_async_wide port map(coeff_m(1)(0),coeff_m(1)(1),coeff_m(1)(2),coeff_m(1)(3),coeff_m(1)(4),coeff_m(1)(5),coeff_m(1)(6),coeff_m(1)(7),coeff(1)(0),coeff(1)(1),coeff(1)(2),coeff(1)(3),coeff(1)(4),coeff(1)(5),coeff(1)(6),coeff(1)(7));
	br2l_1b:busmacro_xc4v_l2r_async_wide port map(coeff_m(1)(8),coeff_m(1)(9),coeff_m(1)(10),coeff_m(1)(11),coeff_m(1)(12),coeff_m(1)(13),coeff_m(1)(14),coeff_m(1)(15),coeff(1)(8),coeff(1)(9),coeff(1)(10),coeff(1)(11),coeff(1)(12),coeff(1)(13),coeff(1)(14),coeff(1)(15));
	br2l_1c:busmacro_xc4v_l2r_async_wide port map(coeff_m(1)(16),coeff_m(1)(17),coeff_m(1)(18),coeff_m(1)(19),coeff_m(1)(20),coeff_m(1)(21),coeff_m(1)(22),coeff_m(1)(23),coeff(1)(16),coeff(1)(17),coeff(1)(18),coeff(1)(19),coeff(1)(20),coeff(1)(21),coeff(1)(22),coeff(1)(23));
	
	
	--koeff2
	br2l_2:busmacro_xc4v_l2r_async_wide port map(coeff_m(2)(0),coeff_m(2)(1),coeff_m(2)(2),coeff_m(2)(3),coeff_m(2)(4),coeff_m(2)(5),coeff_m(2)(6),coeff_m(2)(7),coeff(2)(0),coeff(2)(1),coeff(2)(2),coeff(2)(3),coeff(2)(4),coeff(2)(5),coeff(2)(6),coeff(2)(7));
	br2l_2b:busmacro_xc4v_l2r_async_wide port map(coeff_m(2)(8),coeff_m(2)(9),coeff_m(2)(10),coeff_m(2)(11),coeff_m(2)(12),coeff_m(2)(13),coeff_m(2)(14),coeff_m(2)(15),coeff(2)(8),coeff(2)(9),coeff(2)(10),coeff(2)(11),coeff(2)(12),coeff(2)(13),coeff(2)(14),coeff(2)(15));
	br2l_2c:busmacro_xc4v_l2r_async_wide port map(coeff_m(2)(16),coeff_m(2)(17),coeff_m(2)(18),coeff_m(2)(19),coeff_m(2)(20),coeff_m(2)(21),coeff_m(2)(22),coeff_m(2)(23),coeff(2)(16),coeff(2)(17),coeff(2)(18),coeff(2)(19),coeff(2)(20),coeff(2)(21),coeff(2)(22),coeff(2)(23));
	
	
	
	--koeff3
	br2l_3:busmacro_xc4v_l2r_async_wide port map(coeff_m(3)(0),coeff_m(3)(1),coeff_m(3)(2),coeff_m(3)(3),coeff_m(3)(4),coeff_m(3)(5),coeff_m(3)(6),coeff_m(3)(7),coeff(3)(0),coeff(3)(1),coeff(3)(2),coeff(3)(3),coeff(3)(4),coeff(3)(5),coeff(3)(6),coeff(3)(7));
	br2l_3b:busmacro_xc4v_l2r_async_wide port map(coeff_m(3)(8),coeff_m(3)(9),coeff_m(3)(10),coeff_m(3)(11),coeff_m(3)(12),coeff_m(3)(13),coeff_m(3)(14),coeff_m(3)(15),coeff(3)(8),coeff(3)(9),coeff(3)(10),coeff(3)(11),coeff(3)(12),coeff(3)(13),coeff(3)(14),coeff(3)(15));
	br2l_3c:busmacro_xc4v_l2r_async_wide port map(coeff_m(3)(16),coeff_m(3)(17),coeff_m(3)(18),coeff_m(3)(19),coeff_m(3)(20),coeff_m(3)(21),coeff_m(3)(22),coeff_m(3)(23),coeff(3)(16),coeff(3)(17),coeff(3)(18),coeff(3)(19),coeff(3)(20),coeff(3)(21),coeff(3)(22),coeff(3)(23));
	
END rtl;
--------------------------------------------------
