LIBRARY ieee;
USE ieee.std_logic_1164.all;


---------------------------------------------------
-- SHIFT OPERTATORS
-- Version 2.0
-- Author: Vebjrn Bystrm, NTNU 8.may 2008
---------------------------------------------------

PACKAGE my_package is
	function "sll" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "sll" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;

   function "srl" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "srl" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;

   function "sla" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "sla" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;

   function "sra" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "sra" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;

   function "rol" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "rol" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;

   function "ror" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector;
   function "ror" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector;
END my_package;

PACKAGE BODY my_package is
    FUNCTION "sll" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE ) := (OTHERS => '0');
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(1 TO l'LENGTH - r) := lv(r+1 TO l'LENGTH);
            END IF;
            RETURN result;
        ELSE
            RETURN l SRL (-r);
        END IF;
    END "sll";
    -------------------------------------------------------------------    
    FUNCTION "sll" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE ) := (OTHERS => '0');
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(1 TO l'LENGTH - r) := lv(r+1 TO l'LENGTH);
            END IF;
            RETURN result;
        ELSE
            RETURN l SRL (-r);
        END IF;
    END "sll";

    -------------------------------------------------------------------    
    -- srl
    -------------------------------------------------------------------    
    FUNCTION "srl" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE ) := (OTHERS => '0');
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(r+1 TO l'LENGTH) := lv(1 TO l'LENGTH - r);
            END IF;
            RETURN result;
        ELSE
            RETURN l SLL (-r);
        END IF;
    END "srl";
    -------------------------------------------------------------------    
    FUNCTION "srl" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE ) := (OTHERS => '0');
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(r+1 TO l'LENGTH) := lv(1 TO l'LENGTH - r);
            END IF;
            RETURN result;
        ELSE
            RETURN l SLL (-r);
        END IF;
    END "srl";

    -------------------------------------------------------------------    
    -- sla
    -------------------------------------------------------------------    
    FUNCTION "sla" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE ) := (OTHERS => l(l'RIGHT));
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(1 TO l'LENGTH - r) := lv(r+1 TO l'LENGTH);
            END IF;
            RETURN result;
        ELSE
            RETURN l SRA (-r);
        END IF;
    END "sla";
    -------------------------------------------------------------------    
    FUNCTION "sla" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE ) := (OTHERS => l(l'RIGHT));
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(1 TO l'LENGTH - r) := lv(r+1 TO l'LENGTH);
            END IF;
            RETURN result;
        ELSE
            RETURN l SRA (-r);
        END IF;
    END "sla";

    -------------------------------------------------------------------    
    -- sra
    -------------------------------------------------------------------    
    FUNCTION "sra" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE ) := (OTHERS => l(l'LEFT));
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(r+1 TO l'LENGTH) := lv(1 TO l'LENGTH - r);
            END IF;
            RETURN result;
        ELSE
            RETURN l SLA (-r);
        END IF;
    END "sra";
    -------------------------------------------------------------------    
    FUNCTION "sra" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE ) := (OTHERS => l(l'LEFT));
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            IF r < l'LENGTH THEN
                resultv(r+1 TO l'LENGTH) := lv(1 TO l'LENGTH - r);
            END IF;
            RETURN result;
        ELSE
            RETURN l SLA (-r);
        END IF;
    END "sra";

    -------------------------------------------------------------------    
    -- rol
    -------------------------------------------------------------------    
    FUNCTION "rol" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE );
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
        VARIABLE rv : integer;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            rv := r MOD l'length;
            resultv(1 TO l'LENGTH - rv ) := lv(rv+1 TO l'LENGTH);
            resultv(l'LENGTH - rv + 1 TO l'length) := lv(1 TO rv);
            RETURN result;
        ELSE
            RETURN l ROR (-r);
        END IF;
    END "rol";
    -------------------------------------------------------------------    
    FUNCTION "rol" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE );
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
        VARIABLE rv : integer;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            rv := r MOD l'length;
            resultv(1 TO l'LENGTH - rv ) := lv(rv+1 TO l'LENGTH);
            resultv(l'LENGTH - rv + 1 TO l'LENGTH) := lv(1 TO rv);
            RETURN result;
        ELSE
            RETURN l ROR (-r);
        END IF;
    END "rol";

    -------------------------------------------------------------------    
    -- ror
    -------------------------------------------------------------------    
    FUNCTION "ror" ( l : std_logic_vector;  r : integer ) RETURN std_logic_vector IS
        ALIAS lv : std_logic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_logic_vector ( l'RANGE );
        ALIAS resultv : std_logic_vector ( 1 TO l'LENGTH ) IS result;
        VARIABLE rv : integer;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            rv := r MOD l'length;
            resultv(rv+1 TO l'LENGTH) := lv(1 TO l'LENGTH - rv);
            resultv(1 TO rv) := lv(l'LENGTH - rv + 1 TO l'LENGTH);            
            RETURN result;
        ELSE
            RETURN l ROL (-r);
        END IF;
    END "ror";
    -------------------------------------------------------------------    
    FUNCTION "ror" ( l : std_ulogic_vector; r : integer ) RETURN std_ulogic_vector IS
        ALIAS lv : std_ulogic_vector ( 1 TO l'LENGTH ) IS l;
        VARIABLE result : std_ulogic_vector ( l'RANGE );
        ALIAS resultv : std_ulogic_vector ( 1 TO l'LENGTH ) IS result;
        VARIABLE rv : integer;
    BEGIN
        IF r = 0 OR l'LENGTH = 0 THEN
            RETURN l;
        ELSIF r > 0 THEN
            rv := r MOD l'length;
            resultv(rv+1 TO l'LENGTH) := lv(1 TO l'LENGTH - rv);
            resultv(1 TO rv) := lv(l'LENGTH - rv + 1 TO l'LENGTH);            
            RETURN result;
        ELSE
            RETURN l ROL (-r);
        END IF;
    END "ror";
END;