LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;

library work;
use work.coefficients.all;
USE work.my_package.all; 	

---------------------------------------------------
-- MY FIR filter
-- Version 2.0
-- 11 coefficients 
-- Author: Vebjrn Bystrm, NTNU 8.may 2008
---------------------------------------------------

-------------- ENTITY -----------------------------
ENTITY my_fir IS
    PORT (
        x: IN std_logic_vector(7 downto 0);
		clk, rst : IN std_logic;
        y: OUT std_logic_vector(23 downto 0)
        );
END my_fir;
---------------------------------------------------


-------------- ARCHITECTURE -----------------------

ARCHITECTURE rtl of my_fir IS

TYPE registers IS ARRAY (3 downto 0) 
OF std_logic_vector(7 downto 0);

SIGNAL reg: registers;

type coefreg is array (3 downto 0) of std_logic_vector(23 downto 0);

signal coeff : coefreg;

--------------------PROCESS------------------------	
BEGIN
		
	
		
	process(clk, rst)

	BEGIN
		--- reset registers ---
		if (rst = '1') then 
			reg <= (others => (others => '0'));
		--- update registers ---
		elsif (clk'EVENT and clk ='1') then
			for k in 3 downto 1 loop 
				reg(k) <= reg(k-1);
			end loop;
			reg(0) <= x;
			
		end if;
	END PROCESS;
	
	
	
	y <= coeff(0) + coeff(1) + coeff(2) + coeff(3); 

	
	c0:coeff_0 port map(reg(0), coeff(0));
	c1:coeff_1 port map(reg(1), coeff(1));
	c2:coeff_2 port map(reg(2), coeff(2));
	c3:coeff_3 port map(reg(3), coeff(3));


	
END rtl;
--------------------------------------------------
